/*##########################Nutzfunktionen##########################*/

/*
 * Funktion zur initialen Glassuche -> durchfährt mit dem Servo zweimal den Messbereich
 * ermittelt den Servo-winkel, an dem höchste Messwert erscheint
 * Berechnet Drehwinkel, um den sich die Radachse drehen muss und gibt ihn zurück (als Zeiger)
 * Parameter: Range, in der gesucht werden soll, Zeiger auf den Drehwinkel
 * Return: bool, ob die Messung erfolgreich oder unerfolgreich war.
 */
boolean SucheGlas(int range_min, int range_max, float *winkel, float *abstand)
{
  //Serial.println("SucheGlas");
  float Mess = SensorGlas.messen_mm();      //Messvariable mit initialem Vergleichswert
  int dist_max = 450;                       //maximale Distanz, zu der zuverlässig gemessen werden kann (experimentell ermittelt, 800 lt. Datenblatt)
  float min_mess[] = {dist_max, dist_max};  //Speichert die minimalen Messwerte, initial-Werte sind dist_max
  int min_pos[] = {0, 0};                   //Speichert für hin und zurück die Position des min. Messwertes
  int del = 15;                             //delay, das zwischen Servobewegungen und Messungen liegt
  int pos;                                  //Position des Servos in Grad (90 ist mittig)

  servo.write(range_min);
  delay(1000);

  //Durchfahre die range in beide Richtungen, während zu jedem Grad ein Messwert aufgenommen wird
  //Speichere die geringste Distanz
  for (pos = range_min; pos <= range_max; pos++)
  {
    Mess = SensorGlas.messen_mm_MW();
    //Mess = SensorGlas.messen_mm();
    if (Mess <= dist_max && Mess >= 50 && Mess < min_mess[0])
    {
      min_mess[0] = Mess;
      min_pos[0] = pos;
    }
    Serial.print(pos*4);Serial.print(" ");Serial.println(Mess);
//    if(Mess > 0 && Mess < 800)
//      {Serial.println(Mess);}
    servo.write(pos);
    delay(del);
  }
  for (pos = range_max; pos > range_min; pos--)
  {
    Mess = SensorGlas.messen_mm_MW();
    //Mess = SensorGlas.messen_mm();
    if (Mess <= dist_max && Mess >= 50 && Mess < min_mess[1])
    {
      min_mess[1] = Mess;
      min_pos[1] = pos;
    }
    Serial.print(pos*4);Serial.print(" ");Serial.println(Mess);
//    if(Mess > 0 && Mess < 800)
//      {Serial.println(Mess);}
    servo.write(pos);
    delay(del);
  }

  //Berechnung der Servoposition und des min. Abstandes als Mittelwert aus den beiden gemessenen Minima
  int servo_pos_min = round((min_pos[0] + min_pos[1]) / 2.0); //Servoposition bei min. Abstand
  float abstand_glas = (min_mess[0] + min_mess[1]) / 2.0;   //gemessener Abstand zum Glas

  //Berechnung des Achsenwinkel, um den sich die Roboterachse drehen muss
  int winkel_servo = 90 - servo_pos_min;
  double winkel_radachse;
  winkel_radachse = atan((abstand_glas * sin(winkel_servo / 180.0 * 3.1415)) / (Shotti.returnGeo('l') + abstand_glas * cos(winkel_servo / 180.0 * 3.1415)));

  //Rückgabe-Werte auf entsprechende Zeiger zuweisen
  *winkel = winkel_radachse / 3.1415 * 180;
  *abstand = abstand_glas;

  //Testausgaben
  //  Serial.print("min. Abstand ");Serial.println(abstand_glas);
  //  Serial.print("Servopos ");Serial.println(servo_pos_min);
  //  Serial.print("Servowinkel ");Serial.println(winkel_servo);
  //  Serial.print("winkel Radachse [RAD] ");Serial.println(winkel_radachse);
  //  Serial.print("winkel Radachse [Grad] ");Serial.println(winkel_radachse/3.1415*180);

  //Überprüfung, ob die Messung erfolgreich war
  //Kriterien: Sinniger Abstand, Winkeldifferenz < 15 Grad zwischen den gemessenen Minima
  if (abstand_glas < dist_max && abs(min_pos[0] - min_pos[1]) < 20)
  {
    //Serial.println("Glas gefunden");
    
    //Bringe Servo in min.-Position (optional)
    servo.write(servo_pos_min);
    delay(500);

    return true;
  }
  else
  {
    //Serial.println("kein Glas gefunden");
    return false;
  }
}

/*
 * Funktion zur Öffnung des Magnetventils, Ausgabe von 20cl
 * Parameter: fuellmenge - Wie viel noch in der Flasche enthalten ist
*/
int oeffneVentil(int fuellmenge)
{
  int oeffnezeit_ms = (-0.0102 * fuellmenge + 13.299) * 1000; //Berechne die Zeit, die das Ventil geöffnet sein muss, um 2cl durchlaufen zu lassen

  //Serial.println(oeffnezeit_ms);

  digitalWrite(ventilPin, HIGH); //Öffne Magnetventil
  delay(oeffnezeit_ms);
  digitalWrite(ventilPin, LOW); //Schließe Magnetventil

  return fuellmenge - 20;
}

/*
 * Abgrund-Testfunktion
 * Testet mit dem vorderen Sensor, ob ein Abgrund entdeckt wurde
 * true, wenn Abgrund entdeckt wurde, sonst false
 * Parameter: delta - erlaubte Abweichung in Prozent
 */
boolean TestSensorVorne(int delta)
{
  //int delta = 10; //Erlaubte Abweichung in Prozent
  int Messwert; //Variable für den aktuellen Messwert

  Messwert = SensorVorne.messen();  //Ermittle aktuellen Messwert
  //Serial.println(Messwert);

  if (Messwert - SensorVorne.Schwelle > -delta)
  {
    return false;
  }
  else
  {
    return true;
  }
}

/*
 * Funktion zur Überprüfung des Endschalters
 * true => Schalter zu, false => Schalter offen
 */
boolean Endschalter()
{
  //Testfunktion ohne echten Schalter:
  //return false;
  
  //Richtige Funktion, wenn der Pin korrekt zugewiesen ist:
  if(digitalRead(endschalterPin)==HIGH)
  {
    return true;
  }
  else
  {
    return false;
  }
}
