#include "Arduino.h"

/*
 * Klasse für den Roboter und die Bewegungsfunktionen
 * Enthält Informationen zur Geometrie des Roboters und Fahrfunktionen
 */

class Robot
{
  private:
    int r;            //Radradius in mm
    int R;            //Roboter-Drehradius in mm
    int l;            //Roboter-Länge, von der Achse bis zum Richtungssensor
    float stepwinkel; //Winkel pro step des Motors
    //Deklaration der beiden Steppermotoren
    StepperMotor leftMotor;
    StepperMotor rightMotor;

  public:
    void setup(int _r, int _R, int _l, int _fullturn);  //setup-Funktion zur Initialisierung der Geometrie und der Motoren
    void bewegeGerade(char dir, float v);               //Funktion zum unbegrenzten geradeaus-fahren
    void bewegeGeradeStrecke(char dir, float v, int s); //Funktion zum streckenbegrenzten geradeaus-fahren
    void bewegeStop();                                  //Funktion zum stoppen beider Motoren
    void bewegeDreh(float winkel, float v);             //Funktion zum Drehen um einen bestimmten Winkel
    void bewegeKurve(char dir, int r_Kurve, float v);   //Funktion zum Fahren einer Kurve
    void ausgGeschw();                                  //Gibt die aktuelle Geschwindigkeit aus
    int  returnGeo(char geo);                           //returnt einen Geometriewert des Roboters
    unsigned long returnStepCount(char Motor);          //Gibt den step-count eines Motors zurück
    void motorEnable(int OnOff);
};


/*#################Definitionen der Klassenfunktionen Roboter#####################*/
/*
 * ##################setup-Funktion für die Roboter-Klasse##################
 */
void Robot::setup(int _r, int _R, int _l, int _microstepping)
{
  //Zuweisung der Geometrie-Parameter
  r = _r;
  R = _R;
  l = _l;
  int fullturn = _microstepping * 200;
  stepwinkel = (360.0 / fullturn) / 180 * 3.14159265; //1.8 Grad pro step (bzw. weniger für microstepping)

  //Initialisierung der beiden Stepper-Motoren, Festlegung der Pinbelegung
  leftMotor.setup(2, 3, 4, fullturn);
  rightMotor.setup(5, 6, 7, fullturn);
}

/*
 *##################Funktion zum geradeaus fahren##################
 *dir - v oder r für vorwärts, rückwärts, v zwischen 0 und 1
 */
void Robot::bewegeGerade(char dir, float v)
{
  if (dir == 'v')
  {
    leftMotor.setV(v, 1);
    rightMotor.setV(v, 0);
  }
  else if (dir == 'r')
  {
    leftMotor.setV(v, 0);
    rightMotor.setV(v, 1);
  }
}

/*
 * ##################Funktion zum geradeaus fahren, definierte Strecke##################
 * direction(dir) 0 oder 1, v zwischen 0 und 1, s als Strecke in mm
 */
void Robot::bewegeGeradeStrecke(char dir, float v, int s)
{
  float winkel = (float)s / r;            //zurückzulegender Winkel des Rades
  int steps = round(winkel / stepwinkel); //Steps, um winkel zu erreichen
  //Serial.println(steps);
  int count = 0; //counter für die steps

  if (dir == 'v') //Fahre in die eine Richtung
  {
    while (count < steps)
    {
      count += leftMotor.setV(v, 1);
      rightMotor.setV(v, 0);
    }
  }
  else if (dir == 'r') //oder in die andere Richtung
  {
    while (count < steps)
    {
      count += leftMotor.setV(v, 0);
      rightMotor.setV(v, 1);
    }
  }
}

/*
 * ##################Funktion zum Stoppen beider Motoren##################
 */
void Robot::bewegeStop()
{
  leftMotor.setV(0, 0);
  rightMotor.setV(0, 0);
}

/*
 * ##################Funktion zum Drehen auf der Stelle##################
 * Richtung wird durch den Winkel bestimmt. Negativ für andere Richtung
 * winkel in Grad (negativ und positiv), v in 0 bis 1
 */

void Robot::bewegeDreh(float winkel, float v)
{
  winkel = (winkel * 3.14159265) / 180; //Umrechnung Grad in Bogenmass
  int steps = abs(round((winkel * R) / (r * stepwinkel))); //Berechne nötige Steps (auf 1.8° genau)
  //Serial.println(steps);
  int count[] = {0, 0}; //Schrittzähler

  while (count[0] < steps) //Bis Anzahl Steps erreicht wurde
  {
    if (winkel > 0) //Drehung in die eine Richtung
    {
      count[0] += leftMotor.setV(v, 1);
      count[1] += rightMotor.setV(v, 1);
    }
    else if (winkel < 0) //Drehung in die andere Richtung
    {
      count[0] += leftMotor.setV(v, 0);
      count[1] += rightMotor.setV(v, 0);
    }
  }
}

/*
 * ##################Funktion zum Fahren einer Kurve##################
 * dir r oder l für Rechts- oder Linkskurve, r_Kurve ist Kurvenradius in mm, v ist Geschw. zwischen 0 und 1
 */
void Robot::bewegeKurve(char dir, int r_Kurve, float v)
{
  float vl, vr;
  //float phi = atan2(v/r_Kurve);
  if (dir == 'l')
  {
    vl = tan(v / r_Kurve) * (r_Kurve - R);
    vr = tan(v / r_Kurve) * (r_Kurve + R);
  }
  else if (dir == 'r')
  {
    vr = tan(v / r_Kurve) * (r_Kurve - R);
    vl = tan(v / r_Kurve) * (r_Kurve + R);
  }

  leftMotor.setV(vl, 1);
  rightMotor.setV(vr, 0);
}

/*
 * ##################Funktion zur Ausgabe der Geschwindigkeiten, sollten sie sich ändern##################
 */
void Robot::ausgGeschw()
{
  if (leftMotor.v_p != leftMotor.v || rightMotor.v_p != rightMotor.v)
  {
    Serial.print(leftMotor.v);
    Serial.print("   ");
    Serial.println(rightMotor.v);
  }

  leftMotor.v_p = leftMotor.v;
  rightMotor.v_p = rightMotor.v;
}

/*
 * ##################Funktion zur Rückgabe eines Geometrieparameters##################
 */
int Robot::returnGeo(char geo)
{
  if (geo == 'r') return r;
  else if (geo == 'R') return R;
  else if (geo == 'l') return l;
}

/*
 * ##################Funktion zur Rückgabe der steps eines der Motoren##################
 */
unsigned long Robot::returnStepCount(char Motor)
{
  if(Motor == 'r')
  {
    return leftMotor.outputSteps();
  }
  else if(Motor == 'l')
  {
    return rightMotor.outputSteps();
  }
}

void Robot::motorEnable(int OnOff)
{
  if(OnOff == 0)
  {
    leftMotor.motorEnable(0);
    rightMotor.motorEnable(0);
  }
  else if(OnOff == 1)
  {
    leftMotor.motorEnable(1);
    rightMotor.motorEnable(1);
  }
}

